Package: air
Type: Package
Title: AI Assistant to Write and Understand R Code
Version: 0.2.2
Authors@R: 
    person(given = "Soumya",
           family = "Ray",
           role = c("aut", "cre", "cph"),
           email = "soumya.ray@gmail.com")
Description: An R console utility that lets you ask R related questions to the
    'OpenAI' large language model. It can answer 'how-to()' questions by  
    providing code, and 'whatis()' questions by explaining what given code does.
    You must provision your own key for the 'OpenAI' API 
    <https://platform.openai.com/docs/api-reference>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: httr, keyring, rjson
Suggests: testthat (>= 3.0.0), vcr (>= 0.6.0)
Config/testthat/edition: 3
RoxygenNote: 7.2.3
URL: https://github.com/soumyaray/air
BugReports: https://github.com/soumyaray/air/issues
NeedsCompilation: no
Packaged: 2023-11-17 15:03:35 UTC; soumyaray
Author: Soumya Ray [aut, cre, cph]
Maintainer: Soumya Ray <soumya.ray@gmail.com>
Repository: CRAN
Date/Publication: 2023-11-17 20:10:07 UTC
Built: R 4.2.3; ; 2023-11-17 22:30:37 UTC; unix
