Package: air
Type: Package
Title: AI Assistant to Write and Understand R Code
Version: 0.2.3
Authors@R: c(
    person("Davis", "Vaughan", , "davis@posit.co", role = c("aut", "cre")),
    person("Soumya", "Ray", , "soumya.ray@gmail.com", role = c("aut"))
  )
Description: An R console utility that lets you ask R related questions to the
    'OpenAI' large language model. It can answer how-to questions by  
    providing code, and what-is questions by explaining what given code does.
    You must provision your own key for the 'OpenAI' API 
    <https://platform.openai.com/docs/api-reference>.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
Imports: httr, keyring, result, rjson
Suggests: testthat (>= 3.0.0), vcr (>= 0.6.0)
Config/testthat/edition: 3
RoxygenNote: 7.3.2
URL: https://github.com/soumyaray/air
BugReports: https://github.com/soumyaray/air/issues
NeedsCompilation: no
Packaged: 2025-07-26 14:25:59 UTC; soumyaray
Author: Davis Vaughan [aut, cre],
  Soumya Ray [aut]
Maintainer: Davis Vaughan <davis@posit.co>
Repository: CRAN
Date/Publication: 2025-07-27 06:40:02 UTC
Built: R 4.6.0; ; 2026-04-27 21:58:43 UTC; unix
