# Whitespace is trimmed
a $ b
a @ b
a :: b
a ::: b
1 ? 2
1 ** 2
1 ^ 2
1 : 2

# The symbol is sticky to the operator, there is never a line break immediately after `$`
argument_that_is_quite_quite_quite_quite_long$argument_that_is_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long@argument_that_is_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long::argument_that_is_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long:::argument_that_is_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long?argument_that_is_quite_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long**argument_that_is_quite_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long^argument_that_is_quite_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long:argument_that_is_quite_quite_quite_quite_long

# If the RHS contains other complex expressions, they are allowed to break
argument_that_is_quite_quite_quite_quite_long$argument_that_is_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long@argument_that_is_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long::argument_that_is_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long:::argument_that_is_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long?argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long**argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long^argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long:argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)

# `$` and `@` and regular binary operators are allowed complex expressions on their LHS
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)$argument_that_is_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)@argument_that_is_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)?argument_that_is_quite_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)**argument_that_is_quite_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)^argument_that_is_quite_quite_quite_quite_long
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long):argument_that_is_quite_quite_quite_quite_long

# Chaining of method calls
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)$argument_that_is_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)@argument_that_is_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)?argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)**argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)^argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long):argument_that_is_quite_quite_quite_quite_long(argument_that_is_quite_quite_quite_long)
