1
2.5
1e6
0x123F

# https://github.com/r-lib/tree-sitter-r/pull/198
.1

# Hexadecimal
# `x` vs `X`
0x123
0X123
# Numbers and letters
0xDEAD
0XDEAD
0x1f2F3
0X1f2F3
# `p` vs `P`
0x0p0
0x0P0
0x0p123
0x0P123
# `+` and `-`
0x0p+0
0x0p-0
0x0p+123
0x0p-123
# Decimal point
0x.
0x1.
0x.1
0x1.1
# Decimal point with exponent
0x.p1
0x1.p1
0x.1p1
0x1.1p1